function ControlSpecCB(action)
% ControlSpecCB Control Specification CallBack
%     To understand the program it is necessary to understand system of names of figure
%     fields: Figure has similar fields. The fields are identified by their tags. There
%     is five rows of fields corresponding to five types of requirements: Overshoot,
%     Input, InputDifference, Output, . They correspond to tag endings.
%     In each row there are 3 x 3 fields and one radiobutton fiels. Each field triple 
%     has the same tag, composed from prefix and ending. The prefixes are: Min, Max, (Confidence,
%     and Tolerance), e.g. MaxOvershoot, ToleranceOutputDifference. The fields in triple 
%     difers by Style; there are slider, edit and text styles there. As there are two types
%     of sliders, Tags of sliders showing results are suffixed with Slider, e.g.
%     MaxOvershootSlider.

%In future it'll be suitable to replace all occurances of "difference" by "increment"
%both in this function and in ControlSpec.m too.
switch(action)
case('CreateResult')
   %make results invisible at the beginning; this depends on CreateFcn definitions for 
   %corresponding fields
   set(gcbo,'Visible','off')
case('CreateFigure')
   set(gcf,'UserData',[0])   %count of activated requirements
case('OK')
   ControlPreparation
   close(gcbf)      
case('Modify')
   ControlPreparation
   ModifyNextCB(action)
case('Cancel')
   close(gcbf) 
case('CreateOK')
   TestApplicability(gcbo,'Design')
case('CreateModify')
   TestApplicability(gcbo,'Design')
case('Next')
   figure(findobj('Type','figure','Tag','Designer'))   %show main figure
case('CreateNext')
   TestApplicability(gcbo,'Design')
   set(gcbo,'Visible','off')    %in any case, Next, at the beginning, is invisible
case{'Slider'}
   ModifyNextCB('ButtonDown')
   hide_result
   slider2edit
case{'Edit'}
   ModifyNextCB('ButtonDown')
   hide_result
   edit2slider
case('ControllerC++')
case('Radiobutton')
   if ~hide_result
      if get(gcbo,'Value')
         set(gcbo,'Value',0)
      else
         set(gcbo,'Value',1)
      end         
      return
   end      
   if strcmp(get(gcf,'UserData'),'Results')
      set(gcf,'UserData',[0])   %count of activated requirements
   end   
   ModifyNextCB('ButtonDown')
   tag=get(gcbo,'Tag')
   if get(gcbo,'Value')         %fields were not visible
      make_visible_constraints(tag)
      min_slider_handle=findobj('Tag',['Min' tag 'Slider'],'Style','slider')
      max_slider_handle=findobj('Tag',['Max' tag 'Slider'],'Style','slider')
      set(min_slider_handle,'Visible','off')
      set(max_slider_handle,'Visible','off')
      min_handles=findobj('Tag',['Min' tag],'Style','text')
      max_handles=findobj('Tag',['Max' tag],'Style','text')
      set(min_handles,'Visible','off')
      set(max_handles,'Visible','off')
      if strcmp(tag,'Overshoot')
         set(findobj(gcf,'Style','radiobutton','Tag','Step'),'Visible','on')
      end      
   else                         %fields were visible
      min_handles=findobj('Tag',['Min' tag])
      max_handles=findobj('Tag',['Max' tag])
      %confidence_handles=findobj('Tag',['Confidence' tag])
      %tolerance_handles=findobj('Tag',['Tolerance' tag])
      min_slider_handle=findobj('Tag',['Min' tag 'Slider'],'Style','slider')
      max_slider_handle=findobj('Tag',['Max' tag 'Slider'],'Style','slider')
      set(min_handles,'Visible','off')
      set(max_handles,'Visible','off')
      %set(confidence_handles,'Visible','off')
      %set(tolerance_handles,'Visible','off')      
      set(min_slider_handle,'Visible','off')
      set(max_slider_handle,'Visible','off')
      set(gcf,'UserData',get(gcf,'UserData')-1)
      if strcmp(tag,'Overshoot')
         set(findobj(gcf,'Style','radiobutton','Tag','Step'),'Visible','off','Value',0)
         min_handles=findobj('Tag',['Min' 'Step'])
         max_handles=findobj('Tag',['Max' 'Step'])
         %confidence_handles=findobj('Tag',['Confidence' 'Step'])
         %tolerance_handles=findobj('Tag',['Tolerance' 'Step'])
         min_slider_handle=findobj('Tag',['Min' 'Step' 'Slider'],'Style','slider')
         max_slider_handle=findobj('Tag',['Max' 'Step' 'Slider'],'Style','slider')
         set(min_handles,'Visible','off')
         set(max_handles,'Visible','off')
         %set(confidence_handles,'Visible','off')
         %set(tolerance_handles,'Visible','off')      
         set(min_slider_handle,'Visible','off')
         set(max_slider_handle,'Visible','off')
      end      
      if get(gcf,'UserData')==0
         set(findobj('Tag','Header','Style','text'),'Visible','off')
      end      
   end   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function make_visible_constraints(tag)
%Function make_visible_constraints makes visible a row of constraints
      min_handles=findobj('Tag',['Min' tag])
      max_handles=findobj('Tag',['Max' tag])
      %confidence_handles=findobj('Tag',['Confidence' tag])
      %tolerance_handles=findobj('Tag',['Tolerance' tag])
      set(min_handles,'Visible','on')
      set(max_handles,'Visible','on')
      %set(confidence_handles,'Visible','on')
      %set(tolerance_handles,'Visible','on')
      %results off
      %confidence_handles=findobj('Tag',['Confidence' tag],'Style','text')
      %tolerance_handles=findobj('Tag',['Tolerance' tag],'Style','text')
      %set(confidence_handles,'Visible','off')
      %set(tolerance_handles,'Visible','off')
      %set default values
      if strcmp(tag,'Overshoot')
         set(findobj('Tag',['Min' tag],'Style','edit'),'String','0')
         set(findobj('Tag',['Max' tag],'Style','edit'),'String','1e16')
         set(findobj('Tag',['Min' tag],'Style','slider'),'Value',0)
         set(findobj('Tag',['Max' tag],'Style','slider'),'Value',pi/2)
      else         
         set(findobj('Tag',['Min' tag],'Style','edit'),'String','-1e16')
         set(findobj('Tag',['Max' tag],'Style','edit'),'String','1e16')
         %set(findobj('Tag',['Confidence' tag],'Style','edit'),'String','1')
         %set(findobj('Tag',['Tolerance' tag],'Style','edit'),'String','1')
         set(findobj('Tag',['Min' tag],'Style','slider'),'Value',-pi/2)
         set(findobj('Tag',['Max' tag],'Style','slider'),'Value',pi/2)
         %set(findobj('Tag',['Confidence' tag],'Style','slider'),'Value',1)
         %set(findobj('Tag',['Tolerance' tag],'Style','slider'),'Value',1)
      end
      %maintain counter of requests
      set(gcf,'UserData',get(gcf,'UserData')+1)
      if get(gcf,'UserData')>0
         set(findobj('Tag','Header','Style','text'),'Visible','on')
      end      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function slider2edit
%Function slider2edit translates value of slider to the value of corresponding edit field
value=get(gcbo,'Value')
tag=get(gcbo,'Tag')
%synchronizeAxes(value,strrep(tag,'Slider',''))
%strrep remopves string 'Slider'
if value>0                          %correction for infinity
   value=value-.00000000000001
else
   value=value+.00000000000001
end
value=num2str(tan(value),'%3.1f')         %scaling with tangent and formating
set(findobj('Style','edit','Tag',tag),'String',value)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function edit2slider
%Function edit2slider translates value of slider to the value of corresponding edit field
value=get(gcbo,'String')
tag=get(gcbo,'Tag')
if strcmp(tag,'MinOvershoot')|strcmp(tag,'MaxOvershoot')
   if value<1
      error(['You entered value that is less then one (',value,'). It is not allowed. I''m going to return it to the default. Correct it, to continue!'])
      if strcmp(tag,'MinOvershoot')
         value='1'
      else
         value='1e16'         
      end
      set(gcbo,'String',value)      
   end
end
value=atan(str2num(value))          %scaling with arctangent
set(findobj('Style','slider','Tag',tag),'Value',value)
%synchronizeAxes(value,strrep(tag,'Edit',''))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
%function output_slider(value,tag)
%%Function output_slider translates output value to slider 
%value=atan(str2num(value))          %scaling with arctangent
%set(findobj('Style','slider','Tag',tag),'Value',value,'Visible','on')

%function synchronizeAxes(value,tag)
%%Function synchronizeAxes synchronize Axes, presenting data, with limits specification
%global data input output str Model
%%first make access to axes with data
%designer_figure_handle=findobj('Type','figure','Tag','Designer')
%designer_axes=get(designer_figure_handle,'CurrentAxes')
%axes(designer_axes)
%%then make corresponding graphical synchronization
%line_object=findobj(gca,'Tag',tag,'Type','line')
%set(line_object,'YData',[value value],'Visible','on')
%%move corresponding text also
%text_shift=0.1*max(data(input,:))   
%text_object=findobj(gca,'Tag',tag,'Type','text')
%text_position=get(text_object,'Position')
%if isempty(findstr(tag,'Confidence'))
%   text_position(2)=value-text_shift
%else   
%   text_position(2)=value+text_shift
%end
%set(text_object,'Position',text_position,'Visible','on')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function ControlPreparation
%Function ControlPreparation reads Control parameters and processes them
global data des cfinal cons pre

%initialize results
%make results visible 

%feed inputs into corresponding structure

umin=get(findobj(gcbf,'Style','edit','Tag','MinInput'),'String')
if isempty(umin) 
   umin=-1e16
else
   umin=str2num(umin)   
end;
uref=get(findobj(gcbf,'Style','edit','Tag','RefInput'),'String')
if isempty(uref) 
   uref=-1e16
else
   uref=str2num(uref)   
end;
umax=get(findobj(gcbf,'Style','edit','Tag','MaxInput'),'String')
if isempty(umax) 
   umax=1e16
else
   umax=str2num(umax)   
end;

%uconf=get(findobj(gcbf,'Style','edit','Tag','ConfidenceInput'),'String')
%if isempty(uconf) 
%   uconf=1
%else
%   uconf=str2num(uconf)   
%end;

%utol=get(findobj(gcbf,'Style','edit','Tag','ToleranceInput'),'String')
%if isempty(utol) 
%   utol=1
%else
%   utol=str2num(utol)   
%end;

dumin=get(findobj(gcbf,'Style','edit','Tag','MinInputDifference'),'String')
if isempty(dumin) 
   dumin=-1e16
else
   dumin=str2num(dumin)   
end;
dumax=get(findobj(gcbf,'Style','edit','Tag','MaxInputDifference'),'String')
if isempty(dumax) 
   dumax=1e16
else
   dumax=str2num(dumax)   
end;

%duconf=get(findobj(gcbf,'Style','edit','Tag','ConfidenceInputDifference'),'String')
%if isempty(duconf) 
%   duconf=1
%else
%   duconf=str2num(duconf)   
%end;

%dutol=get(findobj(gcbf,'Style','edit','Tag','ToleranceInputDifference'),'String')
%if isempty(dutol) 
%   dutol=1
%else
%   dutol=str2num(dutol)   
%end;

ovrmax=get(findobj(gcbf,'Style','edit','Tag','MaxOvershoot'),'String')
if isempty(ovrmax) 
   ovrmax=1e16
else
   ovrmax=str2num(ovrmax)   
end;

ovrmin=get(findobj(gcbf,'Style','edit','Tag','MaxOvershoot'),'String')
if isempty(ovrmin) 
   ovrmin=1e16
else
   ovrmin=str2num(ovrmin)   
end;

%ovrtol=get(findobj(gcbf,'Style','edit','Tag','ToleranceOvershoot'),'String')
%if isempty(ovrtol) 
%   ovrtol=1
%else
%   ovrtol=str2num(ovrtol)   
%end;

stepmax=get(findobj(gcbf,'Style','edit','Tag','MaxOvershoot'),'String')
if isempty(stepmax) 
   stepmax=1e16
else
   stepmax=str2num(stepmax)   
end;

stepmin=get(findobj(gcbf,'Style','edit','Tag','MaxOvershoot'),'String')
if isempty(stepmin) 
   stepmin=1e16
else
   stepmin=str2num(stepmin)   
end;

%steptol=get(findobj(gcbf,'Style','edit','Tag','ToleranceOvershoot'),'String')
%if isempty(steptol) 
%   steptol=1
%else
%   steptol=str2num(steptol)   
%end;

ymin=get(findobj(gcbf,'Style','edit','Tag','MinOutput'),'String')
if isempty(ymin) 
   ymin=-1e16
else
   ymin=str2num(ymin)   
end;
yref=get(findobj(gcbf,'Style','edit','Tag','RefOutput'),'String')
if isempty(yref) 
   yref=-1e16
else
   yref=str2num(yref)   
end;
ymax=get(findobj(gcbf,'Style','edit','Tag','MaxOutput'),'String')
if isempty(ymax) 
   ymax=1e16
else
   ymax=str2num(ymax)   
end;

%yconf=get(findobj(gcbf,'Style','edit','Tag','ConfidenceOutput'),'String')
%if isempty(yconf) 
%   yconf=1
%else
%   yconf=str2num(yconf)   
%end;

dymin=get(findobj(gcbf,'Style','edit','Tag','MinOutputDifference'),'String')
if isempty(dymin) 
   dymin=-1e16
else
   dymin=str2num(dymin)   
end;
dymax=get(findobj(gcbf,'Style','edit','Tag','MaxOutputDifference'),'String')
if isempty(dymax) 
   dymax=1e16
else
   dymax=str2num(dymax)   
end;

%dyconf=get(findobj(gcbf,'Style','edit','Tag','ConfidenceOutputDifference'),'String')
%if isempty(dyconf) 
%   dyconf=1
%else
%   dyconf=str2num(dyconf)   
%end;

%up=0;          %not available to user, here from some historical reasons [:JR]
%quini = .01;   % initial guess for input penalization 


%if get(findobj(gcbf,'Style','radiobutton','Tag','Setpoint'),'Value') & ~isempty(setpoint)
%else   

%   setpoint=[0 0; 1e16 0]
%end

cons=cellvect(1)       %creation of cell vector with fictive member (to avoid problems)
ii=1
radiobutton_handle=findobj(gcf,'Style','radiobutton','Tag','Input')
cons=setcell(cons,ii,'u')
ii=ii+1
if get(radiobutton_handle,'Value')         %constraint was required
   cons=setcell(cons,ii,[umin uref umax])
else
   cons=setcell(cons,ii,[uref])
end
ii=ii+1

radiobutton_handle=findobj(gcf,'Style','radiobutton','Tag','InputDifference')
if get(radiobutton_handle,'Value')         %constraint was required
   cons=setcell(cons,ii,'du')
   ii=ii+1
   cons=setcell(cons,ii,[dumin dumax])
   ii=ii+1
end
radiobutton_handle=findobj(gcf,'Style','radiobutton','Tag','Overshoot')
if get(radiobutton_handle,'Value')         %constraint was required
   cons=setcell(cons,ii,'ovr')
   ii=ii+1
   cons=setcell(cons,ii,[ovrmin ovrmax stepmin stepmax])
   ii=ii+1
end
%radiobutton_handle=findobj(gcf,'Style','radiobutton','Tag','Step')
%if get(radiobutton_handle,'Value')         %constraint was required
%   cons=setcell(cons,ii,'step')
%   ii=ii+1
%   cons=setcell(cons,ii,[])
%   ii=ii+1
%end

radiobutton_handle=findobj(gcf,'Style','radiobutton','Tag','Output')
cons=setcell(cons,ii,'y')
ii=ii+1
if get(radiobutton_handle,'Value')         %constraint was required
   cons=setcell(cons,ii,[ymin yref ymax])
else
   cons=setcell(cons,ii,[yref])
end
ii=ii+1

radiobutton_handle=findobj(gcf,'Style','radiobutton','Tag','OutputDifference')
if get(radiobutton_handle,'Value')         %constraint was required
   cons=setcell(cons,ii,'dy')
   ii=ii+1
   cons=setcell(cons,ii,[dymin dymax])
   ii=ii+1
end

if get(findobj(gcf,'Tag','ControllerC++'),'Value')
   %create C++
end   

drawnow
PropagateChanges('Design')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function hide=hide_result
%Function hide_result hides displayed results of performance prediction - value ranges

if strcmp(get(gcf,'UserData'),'Results')
   answer=questdlg('Do you really want to change constraints? It''ll delete displayed results.')
   switch answer
      case 'Yes'
         hide=1
      case {'No', 'Cancel'}
         hide=0
         return
      end %switch
   
   set(gcf,'UserData',[0])   %count of activated requirements

   set(findobj('Style','text','Tag',['Min' 'Input']),'Visible','off')
   set(findobj('Style','text','Tag',['Max' 'Input']),'Visible','off')
   set(findobj('Style','slider','Tag',['Min' 'Input' 'Slider']),'Visible','off')
   set(findobj('Style','slider','Tag',['Max' 'Input' 'Slider']),'Visible','off')

   set(findobj('Style','text','Tag',['Min' 'InputDifference']),'Visible','off')
   set(findobj('Style','text','Tag',['Max' 'InputDifference']),'Visible','off')
   set(findobj('Style','slider','Tag',['Min' 'InputDifference' 'Slider']),'Visible','off')
   set(findobj('Style','slider','Tag',['Max' 'InputDifference' 'Slider']),'Visible','off')

   set(findobj('Style','text','Tag',['Min' 'Overshoot']),'Visible','off')
   set(findobj('Style','text','Tag',['Max' 'Overshoot']),'Visible','off')
   set(findobj('Style','slider','Tag',['Min' 'Overshoot' 'Slider']),'Visible','off')
   set(findobj('Style','slider','Tag',['Max' 'Overshoot' 'Slider']),'Visible','off')

   set(findobj('Style','text','Tag',['Min' 'Step']),'Visible','off')
   set(findobj('Style','text','Tag',['Max' 'Step']),'Visible','off')
   set(findobj('Style','slider','Tag',['Min' 'Step' 'Slider']),'Visible','off')
   set(findobj('Style','slider','Tag',['Max' 'Step' 'Slider']),'Visible','off')

   set(findobj('Style','text','Tag',['Min' 'Output']),'Visible','off')
   set(findobj('Style','text','Tag',['Max' 'Output']),'Visible','off')
   set(findobj('Style','slider','Tag',['Min' 'Output' 'Slider']),'Visible','off')
   set(findobj('Style','slider','Tag',['Max' 'Output' 'Slider']),'Visible','off')

   set(findobj('Style','text','Tag',['Min' 'OutputDifference']),'Visible','off')
   set(findobj('Style','text','Tag',['Max' 'OutputDifference']),'Visible','off')
   set(findobj('Style','slider','Tag',['Min' 'OutputDifference' 'Slider']),'Visible','off')
   set(findobj('Style','slider','Tag',['Max' 'OutputDifference' 'Slider']),'Visible','off')   
end

