function DataSpecCB(action)
% DataSpecCB Data Specification CallBack
global input output data des
global line_object
tag=get(gcbo,'tag')  %get tag value of control identifying other controls (row)
switch(action)
   case{'len','sigma','seed','delay','input','output'}
      ModifyNextCB('ButtonDown')
   case('figure_creation')
   case('OK')
      DataPreparation('generated')
      close(gcbf) 
   case('Modify')
      DataPreparation('generated')      
      ModifyNextCB(action)
   case('SavedData')                    %solving usage of saved data
      DataPreparation('saved')          %case of saved data
   case('Cancel')   
      close(gcbf)
   case('Next')      
      ModifyNextCB(action)
   %following 3 cases control movement of limit lines: UG solution  
   case('StartMove')
      line_object=gcbo
      set(line_object,'LineWidth',1)
      set(gcbf,'WindowButtonMotionFcn','DataSpecCB MoveLine')
      set(gcbf,'WindowButtonUpFcn','DataSpecCB StopMove')
      DataSpecCB MoveLine
   case('MoveLine')   
      currentPoint=get(gca,'CurrentPoint')
      currentPoint=currentPoint(1,2)
      set(line_object,'YData',[currentPoint currentPoint])
      %move corresponding text also
      text_shift=0.1*max(data(input,:))   
      text_tag=get(line_object,'Tag')
      text_object=findobj(gcbf,'Tag',text_tag,'Type','text')
      text_position=get(text_object,'Position')
      if isempty(findstr(text_tag,'Upper'))
         text_position(2)=currentPoint-text_shift
      else         
         text_position(2)=currentPoint+text_shift
      end
      set(text_object,'Position',text_position)
      projectionIntoControlSpec(text_tag,currentPoint)
   case('StopMove')   
      set(gcbf,'WindowButtonMotionFcn','')
      set(gcbf,'WindowButtonUpFcn','')
      set(line_object,'LineWidth',0.5)
   end
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function projectionIntoControlSpec(tag,value)
%Function projectionIntoControlSpec projects limit change into ControlSpec
control_object=findobj('Name','Control Design Specification','Type','figure')
if ~isempty(control_object)
   set(findobj(control_object,'Tag',tag,'Style','edit'),'String',num2str(value))
   set(findobj(control_object,'Tag',tag,'Style','slider'),'Value',value)
end
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function DataPreparation(source)
%Function DataPreparation reads Data parameters and plots Data in Designer figure
global input output data des str frg pri Pri Model pre
%input=[];output=[];data=[];des=[];frg=[];Model=[];   %all is dependent on data and must be changed


%the first data row is y, second u
if strcmp(source,'generated')
	len=get(findobj(gcbf,'Style','edit','Tag','Len'),'String')
	if isempty(len)
   	warndlg('Number of signal items must be specified. Specify or Close!','Warning')
	   return
	else      
   	len=str2num(len)
	end;
   
	sigma=get(findobj(gcbf,'Style','edit','Tag','Sigma'),'String')
	if isempty(sigma) 
   	sigma=0.01
	else      
   	sigma=str2num(sigma)      
	end;

	seed=get(findobj(gcbf,'Style','edit','Tag','Seed'),'String')
	if isempty(seed) 
   	seed=RANDN('seed')   
	else      
   	seed=str2num(seed)
	end;

	offset=get(findobj(gcbf,'Style','edit','Tag','Offset'),'String')
	if isempty(offset) 
   	offset=0
	else      
   	offset=str2num(offset)
	end;

	delay=get(findobj(gcbf,'Style','edit','Tag','Delay'),'String')
	if isempty(delay) 
   	delay=0
	else      
   	delay=str2num(delay)
	end;

	output=get(findobj(gcbf,'Style','edit','Tag','Output'),'String')
	if isempty(output) 
   	output=1
	else      
   	output=str2num(output)
	end;

	input=get(findobj(gcbf,'Style','edit','Tag','Input'),'String')
	if isempty(input) 
   	input=2
	else      
   	input=str2num(input)
	end;

	if (input==1&output==2)|(input==2&output==1)   %OK
	else
   	ERRORDLG(['Input and/or output is not correctly specified.'],'IO specification error','replace')   
	end
   
   data=testdata(len,sigma,seed,offset,delay);
   figure_handle=gcf
end
if strcmp(source,'saved')
   [file_name,file_path]=uigetfile('*.mat','Select file to open for saved data')
	if (file_name==0)
   	return
   end         
   load([file_path,file_name])   %it is expected that data  was saved
   size_data=size(data)
   len=size_data(2)
   figure_handle=findobj('Type','figure','Name','Data Specification')
   figure(figure_handle)
   
   set(findobj(figure_handle,'Style','edit','Tag','Len'),'String',num2str(len))
	set(findobj(figure_handle,'Style','edit','Tag','Sigma'),'String','')
	set(findobj(figure_handle,'Style','edit','Tag','Seed'),'String','')
   set(findobj(figure_handle,'Style','edit','Tag','Offset'),'String','')
   input=2
   output=1
   set(findobj(figure_handle,'String','Modify'),'Visible','off')   %it needs a special solution
   set(findobj(figure_handle,'String','Next'),'Visible','on')

end
des='oi'
if input==1&output==2
   data=[data(2,:);data(1,:)];
   des='io'
end
%designer_figure_handle=findobj('Type','figure','Tag','Designer')
%designer_axes=get(designer_figure_handle,'CurrentAxes')
%axes(designer_axes)
lines=plot(data')     %transposed data
XLabel('t [step]')
YLabel('u, y')
Title('Controlled System Data')   
color_marker1=findobj(figure_handle,'Style','radiobutton','Tag','y')
color_marker2=findobj(figure_handle,'Style','radiobutton','Tag','u')
line_width=0.5+5*len/1000
set(lines(output),'LineWidth',line_width)
set(color_marker1,'BackgroundColor',get(lines(output),'Color'))
set(color_marker2,'BackgroundColor',get(lines(input),'Color'))
set(color_marker1,'Visible','on')
set(color_marker2,'Visible','on')
input_object=findobj(gcbf,'Tag','Input')
set(input_object,'Visible','on')
output_object=findobj(gcbf,'Tag','Output')
set(output_object,'Visible','on')

minim=0.8*min(data(input,:))
lower_input_line=line([0 len],[minim minim])
set(lower_input_line,...
   'ButtonDownFcn','DataSpecCB StartMove',...
   'Color',get(lines(input),'Color'),...
   'LineStyle',':',...
   'Tag','LowerInput',...
   'Visible','off')

maxim=0.8*max(data(input,:))
text_shift=maxim/8
text_object=text(7*len/10,minim-text_shift,'lower input limit')
set(text_object,'Tag','LowerInput',...
   'Visible','off')
upper_input_line=line([0 len],[maxim maxim])
set(upper_input_line,...
   'ButtonDownFcn','DataSpecCB StartMove',...
   'Color',get(lines(input),'Color'),...
   'LineStyle',':',...
   'Tag','UpperInput',...
   'Visible','off')
text_object=text(len/10,maxim+text_shift,'upper input limit')
set(text_object,'Tag','UpperInput',...
   'Visible','off')

minim=0.8*min(data(output,:))
lower_output_line=line([0 len],[minim minim])
set(lower_output_line,...
   'ButtonDownFcn','DataSpecCB StartMove',...
   'Color',get(lines(output),'Color'),...
   'LineStyle',':',...
   'Tag','LowerOutput',...
   'Visible','off')
text_object=text(5*len/10,minim-text_shift,'lower output limit')
set(text_object,'Tag','LowerOutput',...
   'Visible','off')

maxim=0.8*max(data(output,:))
upper_output_line=line([0 len],[maxim maxim])
set(upper_output_line,...
   'ButtonDownFcn','DataSpecCB StartMove',...
   'Color',get(lines(output),'Color'),...
   'LineStyle',':',...
   'Tag','UpperOutput',...
   'Visible','off')
text_object=text(3*len/10,maxim+text_shift,'upper output limit')
set(text_object,'Tag','UpperOutput',...
   'Visible','off')

drawnow
PropagateChanges('Data')