function message=MainLoopKB
% MainLoopKB provides Designer control for Main loop. It is based on knowleedge rules.
global frg cons perf

expert_uncertainty=0.1      % constant that can be (later) set by expert user
expert_being_too=1.5        % constant that can be (later) set by expert user for being too low/high
better_information=0
can_increase=0
ii=1
jj=1
separator='\n'
message=[separator]
message_R=[]

design_figure=findobj('Type','figure','Name','Control Design Specification')
figure(design_figure)                      %make it activa

input_specified=get(findobj(design_figure,'Style','radiobutton','Tag','Input'),'Value')
inputDifference_specified=get(findobj(design_figure,'Style','radiobutton','Tag','InputDifference'),'Value')
output_specified=get(findobj(design_figure,'Style','radiobutton','Tag','Output'),'Value')
outputDifference_specified=get(findobj(design_figure,'Style','radiobutton','Tag','OutputDifference'),'Value')
overshoot_specified=get(findobj(design_figure,'Style','radiobutton','Tag','Overshoot'),'Value')
%Rule 0
if output_specified
   if range_length(perf,'sry') <=range_cons_length(cons,'y')
      output_range_lower=1
   else
      output_range_lower=0
   end
end
   
if outputDifference_specified      
   if range_length(perf,'srdy')<=range_cons_length(cons,'dy')
      outputDifference_range_lower=1
   else
      outputDifference_range_lower=0
   end
end

if (output_specified & outputDifference_specified & ((output_range_lower & ~outputDifference_range_lower) | (~output_range_lower & outputDifference_range_lower)))
   conflict=1   
   message=[message num2str(ii),'. There are conflicting requirements on output. ',separator]
   message_R=[message_R num2str(jj),'. Recommendation: You can increase the length of range of input or input increments. ',separator]
   ii=ii+1
   jj=jj+1
else
   conflict=0
end
%Rule 1
if ~ conflict & ((output_specified & ~output_range_lower) | (outputDifference_specified & ~outputDifference_range_lower))
   message=[message num2str(ii),'. Requirements on output are not satisfied. ',separator]
   ii=ii+1
   can_increase=1
end
%Rule 2
if ~ conflict & ((output_specified & ~output_range_lower) | (outputDifference_specified & ~outputDifference_range_lower))
   message=[message num2str(ii),'. Requirements on output are not fulfilled. ',separator]
   message_R=[message_R num2str(jj),'. Recommendation: You can decrease the length of range of input or input increments. ',separator]
   ii=ii+1
   jj=jj+1
end

%Rule 3
if frg < 0.96
   message=[message num2str(ii),'. Forgetting factor is very low. ',separator]
   ii=ii+1
   better_information=1
end

%Rule 4
if there_are_inputs
else   
   better_information=1
   message=[message num2str(ii),'. There are no inputs in model structure. ',separator]
   ii=ii+1
end

%Rule 5
srsdu=getfield(perf,'srsdu')    %range of standard deviations of input
srsddu=getfield(perf,'srsddu')  %range of standard deviations of input increments
srsdy=getfield(perf,'srsdy')    %range of standard deviations of output

if input_specified
   if range_length(perf,'sru' ) < (srsdu(1) +srsdu(2)) 
      better_information=1
      message=[message num2str(ii),'. There are too large standard deviations of inputs (from ',num2str(srsdu(1),2),' to ',num2str(srsdu(2),2),'). ',separator]
      ii=ii+1
   end
end

if inputDifference_specified
   if range_length(perf,'srdu') < (srsddu(1)+srsddu(2))
      better_information=1
      message=[message num2str(ii),'. There are too large standard deviations of input increments (from ',num2str(srsddu(1),2),' to ',num2str(srsddu(2),2),'). ',separator]
      ii=ii+1
   end   
end

if output_specified
   if range_length(perf,'sry' ) < (srsdy(1) +srsdy(2))
      better_information=1
      message=[message num2str(ii),'. There are too large standard deviations of outputs (from ',num2str(srsdy(1),2),' to ',num2str(srsdy(2),2),'). ',separator]
      ii=ii+1
   end
end

%Rule 6
sru=getfield(perf,'sru')        %range of inputs
srmeu=getfield(perf,'srmeu')    %range of mean values of input

srdu=getfield(perf,'srdu')      %range of input increments 
srmedu=getfield(perf,'srmedu')  %range of mean values of input increments

sry=getfield(perf,'sry')        %range of outputs
srmey=getfield(perf,'srmey')    %range of mean values of output

srdy=getfield(perf,'srdy')      %range of output increments
%srmedy=getfield(perf,'srmedy')  %range of mean values of output increments
k=1.2;
if input_specified
   if (sru(2) >=k*(srmeu(2) +2*srsdu(2))) | (sru(1) <=k*(srmeu(1) -2*srsdu(1))) 
      better_information=1
      message=[message num2str(ii),'. Significant outliers occured in distribution of input. ',separator]
      ii=ii+1
   end
end

if inputDifference_specified
   if (srdu(2)>=k*(srmedu(2)+2*srsddu(2))) | (srdu(1)<=k*(srmedu(1)-2*srsddu(1)))
      better_information=1
      message=[message num2str(ii),'. Significant outliers occured in distribution of input increments. ',separator]
      ii=ii+1
   end
end

if output_specified
   if (sry(2) >=k*(srmey(2) +2*srsdy(2))) | (sry(1) <=k*(srmey(1) -2*srsdy(1)))
      better_information=1
      message=[message num2str(ii),'. Significant outliers occured in distribution of output. ',separator]
      ii=ii+1
   end
end

%Rule 7
bad_overshoot=0
if overshoot_specified
   srovrs=getfield(perf,'srovrs')  %range of overshoots
   cnovrs=getlist(cons,'ovr')    %required range of overshoots
   if cnovrs(1)<0
      bad_overshoot=1
      message=[message num2str(ii),'. Overshoot is negative. It is bad. ',separator]
      ii=ii+1
      better_information=1
   end   
end


%Rule 8
if ~bad_overshoot & overshoot_specified
   if srovrs(2)<(1-expert_uncertainty)*cnovrs(2)
      message=[message num2str(ii),'. Upper range limit of overshoot is lower then required one. ',separator]
      ii=ii+1
      can_increase=1
   end

%Rule 9
   if srovrs(2)>(1+expert_uncertainty)*cnovrs(2)
      message=[message num2str(ii),'. Upper range limit of overshoot is higher then required one. ',separator]
      message_R=[message_R num2str(jj),'. Recommendation: You can decrease the length of range of input or input increments. ',separator]
      ii=ii+1
      jj=jj+1
   end
end

%Rule 10

if input_specified
   sru=getfield(perf,'sru')        %range of inputs
   srumin=getfield(perf,'srumin')  %range of minimum inputs in step response
   srumax=getfield(perf,'srumax')  %range of maximum inputs in step response
   if srumin(1)<1/2*((sru(2)+sru(1))-expert_being_too*(sru(2)-sru(1)))
      message=[message num2str(ii),'. Minimal input (',num2str(srumin(1),2),') is too low compared to lower input interval bound(',num2str(sru(1),2),'). ',separator]
      message_R=[message_R num2str(jj),'. Recommendation: You can decrease the step. ',separator]
      ii=ii+1
      jj=jj+1
   end
   if srumax(2)>1/2*((sru(2)+sru(1))+expert_being_too*(sru(2)-sru(1)))
      message=[message num2str(ii),'. Maximal input (',num2str(srumax(2),2),') is too high compared to upper input interval bound (',num2str(sru(2),2),'). ',separator]
      message_R=[message_R num2str(jj),'. Recommendation: You can decrease the step. ',separator]
      ii=ii+1
      jj=jj+1
   end
end

if better_information
   message_R=[message_R num2str(jj),'. Recommendation: You should provide better information (data or knowledge or max prior model structure). ',separator]
   jj=jj+1
end
if can_increase
   message_R=[message_R num2str(jj),'. Recommendation: You can increase the length of range of input or input increments. ',separator]
   jj=jj+1
end

if ~strcmp(message,'\n') | ~isempty(message_R)
   message=[message, separator, message_R]   %concatenation of state and recomendation parts of message
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function range_len=range_length(constraints, range_name)
%range_length computes the length of range interval in the given constraints structure
range=getfield(constraints, range_name)
range_len=range(2)-range(1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function range_len=range_cons_length(constraints, range_name)
%range_length computes the length of range interval in the given constraints structure
range=getlist(constraints, range_name)
if size(range) == 3
   range_len=range(3)-range(1)
else
   range_len=range(2)-range(1)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function there_are=there_are_inputs
%function checks whether there are inputs in Model structure
global des str 

input=findstr(des,'i')

str_size=size(str)
str_size=str_size(1,2)
for ii=1:str_size
   if str(1,ii)==input
      there_are=1
      return
   end
end
there_are=0


