function PrModStrSpecCB(action)
% PrModStrSpecCB Prior Model Structure CallBack
% often used variable is tag_extension: '_N' ... table specifying prior structure
%                                       '_E' ... table specifying estimated structure
global str maxstr
switch(action)
case('Add')                   %add columns corresponding to delays   
   TableBaseVisible('off','_E')
   DeleteOldFields('_E')
   %construct matrix for Model Structure
   detailed=findobj(gcbf,'Tag','More detailed')
   if get(detailed,'Value')==1
      TableBaseVisible('on','_N')
      PrepareTable('_N',0,maxstr)
   end
   ModifyNextCB('ButtonDown')
   set(gcf,'UserData','prior')   %re-initialize for processing of prior structure
case('Details')
   TableBaseVisible('off','_E')
   DeleteOldFields('_E')
   if ~get(gcbo,'Value')      %not detailed
      DeleteOldFields('_N')
      TableBaseVisible('off','_N')
   else
      TableBaseVisible('on','_N')
      PrepareTable('_N',0,maxstr)
   end
   ModifyNextCB('ButtonDown')
   set(gcf,'UserData','prior')   %re-initialize for processing of prior structure
case('edit')
   ModifyNextCB('ButtonDown')
   set(gcf,'UserData','prior')   %re-initialize for processing of prior structure
   TableBaseVisible('off','_E')
   DeleteOldFields('_E')
case('Slider')
   PrepareTable('_N',1,maxstr)
case('Slider_E')
   PrepareTable('_E',1,str)
case('OK')
   StructurePreparation
   close(gcbf) 
case('Modify')
   ModifyNextCB(action)
   if strcmp(get(gcf,'UserData'),'prior')
      StructurePreparation
      set(gcf,'UserData','estimated')
   end   
   %figure(findobj('Tag','Designer','Type','figure'))   
case('Cancel')
   close(gcbf) 
case('CreateOK')
   TestApplicability(gcbo,'Structure')
   set(gcf,'UserData','prior')   %initialize for processing of prior structure
case('CreateModify')
   TestApplicability(gcbo,'Structure')
case('Next')
   ModifyNextCB(action)
case('CreateNext')
   TestApplicability(gcbo,'Structure')
   set(gcbo,'Visible','off')    %in any case, at the beginning invisible
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function TableBaseVisible(visibility,tag_extension)
%Function TableBaseVisible controls visibility of base elements of tables      
%parameters:
%     visibility

%turn first texts
texts=findobj(gcbf,'Tag',['First_texts' tag_extension])
set(texts,'Visible',visibility)     

%turn first column
objects(2)=findobj(gcbf,'Tag',['Delay_0' tag_extension])
objects(1)=findobj(gcbf,'Tag',['Output_0' tag_extension])
objects(3)=findobj(gcbf,'Tag',['Input_0' tag_extension])
set(objects,'Visible',visibility)

slider=findobj(gcbf,'Style','slider','Tag',['Slider' tag_extension])
set(slider,'Visible',visibility)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function struct=StrDetermination
%Function StrDetermination reads Structure parameters and determines structure
global input output
MinInputDelay_object=findobj(gcbf,'Tag','MinInputDelay')
MinInputDelay=get(MinInputDelay_object,'String')   
MinInputDelay=str2num(MinInputDelay)       
if isempty(MinInputDelay)
   MinInputDelay=0
end
MaxInputDelay_object=findobj(gcbf,'Tag','MaxInputDelay')
MaxInputDelay=get(MaxInputDelay_object,'String')   
MaxInputDelay=str2num(MaxInputDelay)
if isempty(MaxInputDelay)
   MaxInputDelay=0
end
if MinInputDelay>MaxInputDelay
   warndlg('Min input delay should not be greater then max input delay.','Warning')
end
MinOutputDelay_object=findobj(gcbf,'Tag','MinOutputDelay')
MinOutputDelay=get(MinOutputDelay_object,'String')   
MinOutputDelay=str2num(MinOutputDelay)
if isempty(MinOutputDelay)
   MinOutputDelay=0
end
if MinOutputDelay<1
   warndlg('Min output delay should be at least 1.','Warning')   
end
MaxOutputDelay_object=findobj(gcbf,'Tag','MaxOutputDelay')
MaxOutputDelay=get(MaxOutputDelay_object,'String')   
MaxOutputDelay=str2num(MaxOutputDelay)
if isempty(MaxOutputDelay)
   MaxOutputDelay=0
end
if MinOutputDelay>MaxOutputDelay
   warndlg('Min output delay should not be greater then max output delay.','Warning')   
end

%structure determination
k=1
for i1=MinOutputDelay:MaxOutputDelay
   struct(1,k)=output
   struct(2,k)=i1
   k=k+1
end
for i1=MinInputDelay:MaxInputDelay
   struct(1,k)=input
   struct(2,k)=i1
   k=k+1
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function StructurePreparation
%Function StructurePreparation reads Structure parameters and runs eststr and prior
global input output data des maxstr pri Pri Model pre cfinal cons perf
%str=[];frg=[];Model=[];   %all is dependent on Model structure and must be changed
if isempty(data)          %is sensible to continue?
   warndlg('Data was not specified. You should specify it with Data command (in the Designer main menu). Then you can return to this window.','Warning')
   return
end
order=OrderDetermination
detailed=findobj(gcbf,'Tag','More detailed')
maxstr=[]
if get(detailed,'Value')==1
   k1=1                       %following program structure is given by fixed O/I order 
   for i1=1:order             %for each possible output delay
      object=findobj(gcbf,'Tag',['Output_' num2str(i1) '_N'])   %to recognize output
      if ~isempty(object)
         if get(object,'value')==1
            maxstr(1,k1)=output
            maxstr(2,k1)=i1
            k1=k1+1
         end
      end
   end
   k1=k1-1
   k2=1
   for i1=0:order             %for each possible input delay
      object=findobj(gcbf,'Tag',['Input_' num2str(i1) '_N'])   %to recognize input
      if ~isempty(object)
         if get(object,'value')==1
            maxstr(1,k2+k1)=input
            maxstr(2,k2+k1)=i1
            k2=k2+1
         end         
      end      
   end
   %CheckStructureParameters
else
   maxstr=StrDetermination        %case of not detailed specification
end
if get(findobj(gcbf,'Tag','Offset'),'Value')   %is offset?
   maxstr=[maxstr [0;1]]             %extending str by column for offset
end
if isempty(maxstr)
   warndlg('Prior Model Structure was not (meaningfully) specified.','Warning')
   objects(2)=findobj(gcbf,'Tag','Delay_0_E')
   objects(1)=findobj(gcbf,'Tag','Output_0_E')
   objects(3)=findobj(gcbf,'Tag','Input_0_E')
   set(objects,'Visible','off')   
   DeleteOldFields('_E')
   set(findobj(gcbf,'Tag','Str_empty'),'Visible','on')
   return
else
   set(findobj(gcbf,'Tag','Str_empty'),'Visible','off')   
end   

drawnow
PropagateChanges('Structure')
   

