function PropagateChanges(type)
% function PropagateChanges is common function for propagation of changes.
%     type specifies what type of structure did change; if there is a change, it is
%     propagated by recursive call of PropagateChanges, oriented on following structure.
%     It is also used in functions controlling data input to cover corresponding post-processing.
global data des str frg pri Pri Model frgs frgup cfinal cons pre maxstr data_h1 cons_s data_h2 pri_s pre_h2 ref perfver
switch(type)
case('Data')
   NextToggleButtonCB('DataHandling')      
   if ~isempty(cons)
      %necessary to handle data and constraints again
      PropagateChanges('Design')
   end   
case('Design')                   
	[data_h1, pre] = datahand(data, des, cons);   % data are normalized   
   NextToggleButtonCB('ControlSpecification')
   if ~isempty(str)
      PropagateChanges('Structure')
   end   
case('Structure')
	% control period is estimated, data sampled
	maxper=3
	per = peride(data_h1,des,maxper,maxstr)
	if per>1
   	holder = cellvect('Hold', per);      % requirement to sample
	   data_h2   = datahand(data_h1, holder);  % reduced data
   	pre_h2    = cellvect(pre,holder);       % overall preprocessing
   else
      data_h2 = data_h1;
      pre_h2  = pre
   end
	% constrains are scaled
	cons_s   = scaleit(cons, des, 'cons', pre_h2); % design constraints
   NextToggleButtonCB('StructureSpecification')

	if ~isempty(pri)
      PropagateChanges('Knowledge')
   %else
   %   PropagateChanges('Parameters')      
   end   
case('Knowledge') 
	pri_s=scaleit(pri,des,'pri',pre_h2)
   NextToggleButtonCB('PriorKnowledgeSpecification')
   if ~isempty(str)
      PropagateChanges('Parameters')      
   end   
case('Parameters') 
	%this runs rest of processing that was previously specified
	if isempty(pri)
   	warndlg('Prior Knowledge was not specified. You could specify it with Knowledge command (in the Designer main menu). For the current design, no prior knowledge was used.','Warning')
	   str=eststr('data_h2',des,maxstr)
		NextToggleButtonCB('StructureEstimation')   
	   StructurePresention
	else
      str=eststr('data_h2',des,maxstr,pri_s)
		NextToggleButtonCB('StructureEstimation')   
   	StructurePresention
	   if isempty(str)
   	   return
	   end
   	Pri=prior(des,str,pri_s)         %in prototype no Ini
	   NextToggleButtonCB('PriorKnowledge')   
	end

	% 7) parameters are estimated
 	Model  = estpar('data_h2', des , str);
	if ~isempty(pri)
	   Model = primerge(Model,des,pri_s);   % merge model with prior knowledge
	end
	NextToggleButtonCB('ParameterEstimation')

	%part of Control Design
	cfinal=cdesign(Model,des,cons_s);
	NextToggleButtonCB('ControlDesign')
	ControlDesignPresentation

	%Main loop evaluation works according to its name
	message=MainLoopKB
	if strcmp(message,'\n')
	   msgbox(sprintf('Main loop evaluated as OK! Congratulations.'))
	else
   	msgbox(sprintf(message))
	end   
   if ~isempty(frg)
      PropagateChanges('Forgetting')      
   end   
case('Forgetting')
	if isempty(pri)
   	frg=estfrg('data_h2',des,str)
	else
   	frg=estfrg('data_h2',des,str,Pri,Model)
	end

	% build  controller
	%cmodule = 'ctrl';
   %[des, str, ref]= encreg(Model,des,cfinal,frg,pre_h2,cmodule,Pri); % build controller
   [des, str, ref] = encreg(Model,des,cfinal,frg,pre,0,Model);		%in this way enc controller is built in memory
   %it is strange but reasons are somwhere out of my concerns
	NextToggleButtonCB('ForgettingEstimation')
   if ~isempty(perfver)
      PropagateChanges('Verification')
   end   
case('Verification')
   verification
	NextToggleButtonCB('ControlVerification')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function StructurePresention
%Function StructurePresention presents estimated Structure 
global input output str
figure_handle=findobj('Type','figure','Name','Prior Model Structure Specification')
if isempty(figure_handle)
	open PrModStrSpecOld.fig
	figure_handle=findobj('Type','figure','Name','Prior Model Structure Specification')
else
	figure(figure_handle)
end
figure(figure_handle)                      %make it activa
if isempty(str)
   objects(2)=findobj(figure_handle,'Tag','Delay_0_E')
   objects(1)=findobj(figure_handle,'Tag','Output_0_E')
   objects(3)=findobj(figure_handle,'Tag','Input_0_E')
   set(objects,'Visible','off')
   set(findobj(figure_handle,'Tag','Str_empty'),'Visible','on')
   return
else
   set(findobj(figure_handle,'Tag','Str_empty'),'Visible','off')   
end   
PrepareTable('_E',0,str)

s1=size(str)
offset_object=findobj(figure_handle,'Tag','Offset_E')
set(offset_object,'Visible','on')
if str(1,s1(2))==0         %there is offset in str
   s1(2)=s1(2)-1
   set(offset_object,'Value',1)
else   
   set(offset_object,'Value',0)
end
for i1=1:s1(2)             %for each possible str element insert value in corresponding field
   if str(1,i1)==output
      object=findobj(figure_handle,'Tag',['Output_' num2str(str(2,i1)) '_E'])   %to recognize output
      set(object,'value',1)
   end
   if str(1,i1)==input
      object=findobj(figure_handle,'Tag',['Input_' num2str(str(2,i1)) '_E'])   %to recognize output
      set(object,'value',1)
   end
end
%CheckStructureParameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function error=CheckStructureParameters
%%Function CheckStructureParameters checks structure parameters used in prior knowledge
%global input output des str pri
%max_par=0
%s1=size(str)
%error=0
%if ~isempty(pri)
%   for i1=1:2:size4(pri)
%      if getcell(pri,i1)=='g'   %cell with parameter ranges
%         di=getcell(pri,i1+1)
%         not_considered=1
%         for i1=1:s1(2)
%            if str(1,i1)==di(1) & str(2,i1)==di(2)
%               not_considered=0 %corresponding column is found
%            end                        
%         end
%         if not_considered==1   %nothing was found
%            if des(di(1))=='o'
%               i_o=' (output) '
%            end
%            if des(di(1))=='i'
%               i_o=' (input) '
%            end            
%            text=['Parameter for signal ', num2str(di(1)), i_o, 'with delay ', num2str(di(2)), ' isn''t considered in Model structure. Look into Prior Knowledge Specification figure.']
%            warndlg(text,'Warning')
%            error=1
%         end         
%      end      
%   end   
%end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ControlDesignPresentation
%Function StructurePresention presents estimated Structure 
global des Model cfinal pre_h2 perf
figure_handle=findobj('Type','figure','Name','Control Design Specification')
if isempty(figure_handle)
	open ControlSpecOld.fig
	figure_handle=findobj('Type','figure','Name','Control Design Specification')
else
	figure(figure_handle)
                      %make it activa
end
%disppro                      
%global ENCWORKSPACE
%save PN_enccall_2 ENCWORKSPACE
perf=requsat(Model,des,cfinal,pre_h2);
dispreg(cfinal);
dispperf(perf);
set(figure_handle,'UserData','Results')    %figure is now in results state

radiobutton_handle=findobj(figure_handle,'Style','radiobutton','Tag','Input')
if get(radiobutton_handle,'Value')         %constraint was required
   sru=getfield(perf,'sru')                  %range of inputs
   display_result(sru,'Input');
end

radiobutton_handle=findobj(figure_handle,'Style','radiobutton','Tag','InputDifference')
if get(radiobutton_handle,'Value')         %constraint was required
   srdu=getfield(perf,'srdu')                %range of input increments 
   display_result(srdu,'InputDifference');
end

radiobutton_handle=findobj(figure_handle,'Style','radiobutton','Tag','Overshoot')
if get(radiobutton_handle,'Value')         %constraint was required
   srovrs=getfield(perf,'srovrs')            %range of overshoots
   display_result(srovrs,'Overshoot');
end

radiobutton_handle=findobj(figure_handle,'Style','radiobutton','Tag','Step')
if get(radiobutton_handle,'Value')         %constraint was required
   srumin=getfield(perf,'srumin')            %range of step???
   display_result(srumin,'Step');
end

radiobutton_handle=findobj(figure_handle,'Style','radiobutton','Tag','Output')
if get(radiobutton_handle,'Value')         %constraint was required
   sry=getfield(perf,'sry')                  %range of outputs
   display_result(sry,'Output');
end

radiobutton_handle=findobj(figure_handle,'Style','radiobutton','Tag','OutputDifference')
if get(radiobutton_handle,'Value')         %constraint was required
   srdy=getfield(perf,'srdy')                %range of output increments
   display_result(srdy,'OutputDifference');
end

%srmey=getfield(perf,'srmey')    %range of mean values of output
%srsdy=getfield(perf,'srsdy')    %range of standard deviations of output
%srmeu=getfield(perf,'srmeu')    %range of mean values of input
%srsdu=getfield(perf,'srsdu')    %range of standard deviations of input
%srmedu=getfield(perf,'srmedu')  %range of mean values of input increments
%srsddu=getfield(perf,'srsddu')  %range of standard deviations of input increments
%sroffs=getfield(perf,'sroffs')  %range of offsets
%srumin=getfield(perf,'srumin')  %range of minimum inputs in step responce
%srumax=getfield(perf,'srumax')  %range of maximum inputs in step responce
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
function display_result(value,tag)
%Function display_result translates output value range to text and slider fileds
if value(1)>0                          %correction for infinity
   value(1)=value(1)-.00000000000001
else
   value(1)=value(1)+.00000000000001
end
if value(2)>0                          %correction for infinity
   value(2)=value(2)-.00000000000001
else
   value(2)=value(2)+.00000000000001
end
set(findobj('Style','text','Tag',['Min' tag]),'String',num2str(value(1),'%3.1f'),'Visible','on')
set(findobj('Style','text','Tag',['Max' tag]),'String',num2str(value(2),'%3.1f'),'Visible','on')
set(findobj('Style','slider','Tag',['Min' tag 'Slider']),'Value',atan(value(1)),'Visible','on')
set(findobj('Style','slider','Tag',['Max' tag 'Slider']),'Value',atan(value(2)),'Visible','on')
%?radiobutton_handle=findobj(gcf,'Style','radiobutton','Tag',tag)

