function verification
% function verification provides verification of the designed controller
global des ref perfver systemNo
% 11) controller verification
   uref = ref(1);          % reference values
   yref = ref(2);
   ychn = find(des=='o');  % output is ...
   uchn = find(des=='i');  % input is ...
   chns = [ychn uchn 3];   % setup

% the system:
   [Eth,str]  = system(systemNo,des);
   ssig  = 0.01;        % std of output
   msig = 0;            % std of measurement noise
   t0   = 1000;         % simulation length
   randn('seed', 123);  % fix random generator

% simulation system
Sys      = arxsys0(des,str,Eth,ssig,uref,yref,msig);
u        = uref;         % initial condition for input
setpoint = [yref+zeros(1,t0)];
Y = [];  U = [];         % for signal recording

for t=1:length(setpoint);
       [y, Sys]   = arxsys(Sys, u);       % generate output
       signal(chns) = [y u setpoint(t)];  % complete interface
       Y = [Y,y];   U = [U,u];            % record signal
       signal     = enccall(signal);      % controller
       u          = signal(uchn);         % get input
echo off; end; echo on

% control quality
   perfver = verres(Y-yref,U-uref);
   dispperf(perfver,'verification results');
%%% display %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
y = Y(1:100); u = U(1:100);
uref = 0; yref = 0;
t0 = length(y);
du = u(2:t0) - u(1:t0-1);
dy = y(2:t0) - y(1:t0-1);
echo off % plots
figure_handle=findobj('Type','figure','Tag','Disturbance')
if isempty(figure_handle)
	open Disturbance.fig
	figure_handle=findobj('Type','figure','Tag','Disturbance')
else
   figure(figure_handle)
end

figure_axes=get(figure_handle,'CurrentAxes')
axes(figure_axes)
subplot(2,1,1); plot(y);
grid on
xlabel('time');
ylabel('output');
title('Output disturbance compensation');

subplot(2,1,2); plot(u);
grid on
xlabel('time');
ylabel('input');

figure_handle=findobj('Type','figure','Tag','Increments')
if isempty(figure_handle)
	open Increments.fig
	figure_handle=findobj('Type','figure','Tag','Increments')
else
   figure(figure_handle)
end

figure_axes=get(figure_handle,'CurrentAxes')
axes(figure_axes)
subplot(2,1,1); plot(dy);
grid on
xlabel('time');
ylabel('output');
title('Input and output increments');
subplot(2,1,2); plot(du);
grid on
xlabel('time');
ylabel('input');